/**
 * Simple classes for Matrix-FILE input/output.
 *
 * Copyright - Kirill Gavrilov ' 2008.
 */

#include "matfile.h"

void MatFileReader::computeWidth() {
    if (inFile.fail()) {
        if (debug) {
            std::cout << "computeWidth: inFile.fail()\n";
        }
        return;
    }
    width = 0;
    inFile.seekg (0, std::ios::beg);
    inFile.getline(buffer.body, buffer.size);
    for (unsigned int pos = 0; pos < buffer.size && buffer.body[pos] != '\0'; pos++) {
        if (buffer.body[pos] != ' ' && buffer.body[pos] != '\t') {
            width++;
            for (; pos < buffer.size && buffer.body[pos] != '\0'; pos++) {
                if (buffer.body[pos] == ' ' || buffer.body[pos] == '\t') {
                    break;
                }
            }
        }
    }
    inFile.seekg (0, std::ios::beg);
}

void MatFileReader::computeHeight() {
    if (inFile.fail()) {
        if (debug) {
            std::cout << "computeHeight: inFile.fail()\n";
        }
        return;
    }
    height = 0;
    inFile.seekg (0, std::ios::beg);
    while (!inFile.eof()) {
        inFile.getline(buffer.body, buffer.size);
        if (strlen(buffer.body) > 0) {
            height++;
        }
    }
    inFile.seekg (0, std::ios::beg);
}

MatFileReader::MatFileReader(const char* inFileName, bool inDebug) {
    debug = inDebug;
    fileName = inFileName;
    buffer.size = 4096;
    buffer.body = new char[buffer.size];
    inFile.open(fileName);
    if (!inFile.fail()) {
        computeWidth();
        computeHeight();
        inFile.close();
        status = true;
    } else {
        if (debug) {
            std::cout << "Error: can not open file: " << inFileName << "\n";
        }
        status = false;
    }
}

MatFileReader::~MatFileReader() {
    inFile.close();
    delete [] buffer.body;
}

bool MatFileReader::isFailed() {
    return !status;
}

unsigned int MatFileReader::getWidth() {
    return width;
}

unsigned int MatFileReader::getHeight() {
    return height;
}

bool MatFileReader::getLines(unsigned int lineStart, unsigned int lineEnd, double* values) {
    std::ifstream inFile;
    inFile.open(fileName);
    if (inFile.fail()) {
        if (debug) {
            std::cout << "getLines: inFile.fail()\n";
        }
        return false;
    } else {
        currLine = 0;
        //inFile.seekg (0, std::ios::beg);
        while (!inFile.eof() && currLine < lineStart) {
            inFile.getline(buffer.body, buffer.size);
            if (strlen(buffer.body) > 0) {
                currLine++;
            }
        }
        currIndex = 0;
        while (!inFile.eof() && currIndex < width * (1 + lineEnd - lineStart)) {
            inFile >> values[currIndex++];
        }
        //inFile.seekg (0, std::ios::beg);
        inFile.close();
        return true;
    }
}

SimpleMatFile::SimpleMatFile(const char* fileName) {
    inFile.open(fileName);
}

SimpleMatFile::~SimpleMatFile() {
    inFile.close();
}

bool SimpleMatFile::read(unsigned int count, double* values) {
    if (!inFile.fail()) {
        for(index = 0; !inFile.eof() && index < count; index++) {
            inFile >> values[index];
        }
        return true;
    }
    return false;
}
