/**
 * Simple classes for Matrix-FILE input/output.
 * (2-dimensions matrixes)
 *
 * Copyright - Kirill Gavrilov ' 2008.
 */

#ifndef __MatFileReader_H__
#define __MatFileReader_H__

#include <iostream>
#include <fstream>

/**
 * Class parse input file for matrix size (2-dimensions)
 */
class MatFileReader {

        private:

    std::ifstream inFile;   // file descriptor
    const char* fileName;   // file name for reading
    bool status;            // save last status
    bool debug;             // show debug messages?

    unsigned int width;     // matrix width in file
    unsigned int height;    // matrix height in file

    struct {
        unsigned int size;
        char *body;
    } buffer;               // buffer used for some file read/write operations

    unsigned int currLine;  // temporary variables
    unsigned int currIndex; // temporary variables

    /**
     * Function parse first line and count matrix width
     * (function used for files with unknown matrix configuration).
     */
    void computeWidth();

    /**
     * Function parse whole file and compute matrix height
     * (function used for files with unknown matrix configuration).
     */
    void computeHeight();

        public:

    /**
     * You must put file name with matrix to open.
     * You may check results with <i>isFailed()</i> method.
     * @param inFileName (const char* ) - file name;
     * @param inDebug (bool ) - show debug messages?
     */
    MatFileReader(const char* inFileName, bool inDebug = false);

    ~MatFileReader();

    /**
     * @return false if error file reading
     */
    bool isFailed();

    /**
     * Function return matrix width.
     * @return width (unsigned int ) - matrix width.
     */
    unsigned int getWidth();

    /**
     * Function return matrix height.
     * @return height (unsigned int ) - matrix height.
     */
    unsigned int getHeight();

    /**
     * Function set input matrix with values from start to end lines in file-matrix.
     * @param lineStart (unsigned int ) - line from matrix (FILE) to start;
     * @param lineEnd (unsigned int ) - line from matrix (FILE) to end;
     * @param values (double* ) - buffer for values;
     * @return true if success.
     */
    bool getLines(unsigned int lineStart, unsigned int lineEnd, double* values);

};

/**
 * More simple matrix file reader
 */
class SimpleMatFile{

        private:

    std::ifstream inFile;
    unsigned int index;

        public:

    SimpleMatFile(const char* fileName);
    ~SimpleMatFile();
    bool read(unsigned int count, double* values);

};

#endif //__MatFileReader_H__
